@@ -27,6 +27,7 @@
 #include "ephy-synchronizable-manager.h"
 
 #include <glib/gi18n.h>
+#include <inttypes.h>
 #include <stdio.h>
 
 const SecretSchema *
@@ -188,7 +189,7 @@ get_attributes_table (const char *id,
                       const char *username,
                       const char *username_field,
                       const char *password_field,
-                      double      server_time_modified)
+                      gint64      server_time_modified)
 {
   GHashTable *attributes = secret_attributes_build (EPHY_FORM_PASSWORD_SCHEMA, NULL);
 
@@ -219,7 +220,7 @@ get_attributes_table (const char *id,
   if (server_time_modified >= 0)
     g_hash_table_insert (attributes,
                          g_strdup (SERVER_TIME_MODIFIED_KEY),
-                         g_strdup_printf ("%.2lf", server_time_modified));
+                         g_strdup_printf ("%"PRId64, server_time_modified));
 
   return attributes;
 }
@@ -533,7 +534,7 @@ secret_service_search_cb (SecretService  *service,
     const char *password_field = g_hash_table_lookup (attributes, PASSWORD_FIELD_KEY);
     const char *timestamp = g_hash_table_lookup (attributes, SERVER_TIME_MODIFIED_KEY);
     const char *password = secret_value_get (value, NULL);
-    double server_time_modified;
+    gint64 server_time_modified;
     EphyPasswordRecord *record;
 
     LOG ("Found password record for (%s, %s, %s, %s, %s)",
@@ -549,7 +550,7 @@ secret_service_search_cb (SecretService  *service,
                                        username_field, password_field,
                                        secret_item_get_created (item) * 1000,
                                        secret_item_get_modified (item) * 1000);
-    sscanf (timestamp, "%lf", &server_time_modified);
+    server_time_modified = g_ascii_strtod (timestamp, NULL);
     ephy_synchronizable_set_server_time_modified (EPHY_SYNCHRONIZABLE (record),
                                                   server_time_modified);
     records = g_list_prepend (records, record);
@@ -778,7 +779,7 @@ synchronizable_manager_set_is_initial_sync (EphySynchronizableManager *manager,
   ephy_sync_utils_set_passwords_sync_is_initial (is_initial);
 }
 
-static double
+static gint64
 synchronizable_manager_get_sync_time (EphySynchronizableManager *manager)
 {
   return ephy_sync_utils_get_passwords_sync_time ();
@@ -786,7 +787,7 @@ synchronizable_manager_get_sync_time (EphySynchronizableManager *manager)
 
 static void
 synchronizable_manager_set_sync_time (EphySynchronizableManager *manager,
-                                      double                     sync_time)
+                                      gint64                     sync_time)
 {
   ephy_sync_utils_set_passwords_sync_time (sync_time);
 }
@@ -898,14 +899,14 @@ delete_record_by_id (GList      *records,
   return records;
 }
 
-static GList *
+static GPtrArray *
 ephy_password_manager_handle_initial_merge (EphyPasswordManager *self,
                                             GList               *local_records,
                                             GList               *remote_records)
 {
   EphyPasswordRecord *record;
   GHashTable *dont_upload;
-  GList *to_upload = NULL;
+  GPtrArray *to_upload;
   const char *remote_id;
   const char *remote_origin;
   const char *remote_target_origin;
@@ -915,8 +916,8 @@ ephy_password_manager_handle_initial_merge (EphyPasswordManager *self,
   const char *remote_password_field;
   guint64 remote_timestamp;
   guint64 local_timestamp;
-  double remote_server_time_modified;
-  double local_server_time_modified;
+  gint64 remote_server_time_modified;
+  gint64 local_server_time_modified;
 
   g_assert (EPHY_IS_PASSWORD_MANAGER (self));
 
@@ -927,6 +928,7 @@ ephy_password_manager_handle_initial_merge (EphyPasswordManager *self,
    * same tuple but same tuple does not necessarily mean same ID. This is what
    * our merge logic is based on.
    */
+  to_upload = g_ptr_array_new_with_free_func (g_object_unref);
   dont_upload = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
 
   for (GList *l = remote_records; l && l->data; l = l->next) {
@@ -1006,7 +1008,7 @@ ephy_password_manager_handle_initial_merge (EphyPasswordManager *self,
   for (GList *l = local_records; l && l->data; l = l->next) {
     record = EPHY_PASSWORD_RECORD (l->data);
     if (!g_hash_table_contains (dont_upload, ephy_password_record_get_id (record)))
-      to_upload = g_list_prepend (to_upload, g_object_ref (record));
+      g_ptr_array_add (to_upload, g_object_ref (record));
   }
 
   g_hash_table_unref (dont_upload);
@@ -1014,14 +1016,14 @@ ephy_password_manager_handle_initial_merge (EphyPasswordManager *self,
   return to_upload;
 }
 
-static GList *
+static GPtrArray *
 ephy_password_manager_handle_regular_merge (EphyPasswordManager  *self,
                                             GList               **local_records,
                                             GList                *deleted_records,
                                             GList                *updated_records)
 {
   EphyPasswordRecord *record;
-  GList *to_upload = NULL;
+  GPtrArray *to_upload;
   const char *remote_id;
   const char *remote_origin;
   const char *remote_target_origin;
@@ -1033,6 +1035,8 @@ ephy_password_manager_handle_regular_merge (EphyPasswordManager  *self,
 
   g_assert (EPHY_IS_PASSWORD_MANAGER (self));
 
+  to_upload = g_ptr_array_new_with_free_func (g_object_unref);
+
   for (GList *l = deleted_records; l && l->data; l = l->next) {
     remote_id = ephy_password_record_get_id (l->data);
     record = get_record_by_id (*local_records, remote_id);
@@ -1068,7 +1072,7 @@ ephy_password_manager_handle_regular_merge (EphyPasswordManager  *self,
         local_timestamp = ephy_password_record_get_time_password_changed (record);
         if (local_timestamp > remote_timestamp) {
           /* Local record is newer. Keep it, upload it and delete remote record from server. */
-          to_upload = g_list_prepend (to_upload, g_object_ref (record));
+          g_ptr_array_add (to_upload, g_object_ref (record));
           g_signal_emit_by_name (self, "synchronizable-deleted", l->data);
         } else {
           /* Remote record is newer. Forget local record and store remote record. */
@@ -1089,19 +1093,17 @@ merge_cb (GList    *records,
           gpointer  user_data)
 {
   MergePasswordsAsyncData *data = (MergePasswordsAsyncData *)user_data;
-  GList *to_upload = NULL;
+  GPtrArray *to_upload;
 
   if (data->is_initial)
-    to_upload = ephy_password_manager_handle_initial_merge (data->manager,
-                                                            records,
+    to_upload = ephy_password_manager_handle_initial_merge (data->manager, records,
                                                             data->remotes_updated);
   else
-    to_upload = ephy_password_manager_handle_regular_merge (data->manager,
-                                                            &records,
+    to_upload = ephy_password_manager_handle_regular_merge (data->manager, &records,
                                                             data->remotes_deleted,
                                                             data->remotes_updated);
 
-  data->callback (to_upload, FALSE, data->user_data);
+  data->callback (to_upload, data->user_data);
 
   g_list_free_full (records, g_object_unref);
   merge_passwords_async_data_free (data);
