@@ -104,9 +104,7 @@ popup_cmd_media_in_new_window (GSimpleAction *action,
 static void
 popup_cmd_copy_to_clipboard (EphyWindow *window, const char *text)
 {
-  gtk_clipboard_set_text (gtk_clipboard_get (GDK_NONE),
-                          text, -1);
-  gtk_clipboard_set_text (gtk_clipboard_get (GDK_SELECTION_PRIMARY),
+  gtk_clipboard_set_text (gtk_clipboard_get_default (gdk_display_get_default ()),
                           text, -1);
 }
 
@@ -384,7 +382,7 @@ popup_cmd_search_selection (GSimpleAction *action,
   search_term = g_variant_get_string (parameter, NULL);
   search_url = ephy_embed_utils_autosearch_address (search_term);
   new_embed = ephy_shell_new_tab (ephy_shell_get_default (),
-                                  EPHY_WINDOW (user_data), embed, EPHY_NEW_TAB_APPEND_AFTER);
+                                  EPHY_WINDOW (user_data), embed, EPHY_NEW_TAB_APPEND_AFTER | EPHY_NEW_TAB_JUMP);
   ephy_web_view_load_url (ephy_embed_get_web_view (new_embed), search_url);
   g_free (search_url);
 }
